#include "BadBall.h"
#include <GL/glut.h>
#include "BoundingSphere.h"

CBadBall::CBadBall(const CVector3d& pos, float rad, float dist, CBall* ball)
: CObject(pos)
{
	radius = rad;
	anchor = pos;
	leash = dist;
	bound = new CBoundingSphere(pos, radius);
	target = ball;
	force.Zero();
	velocity.Zero();

	displayList = 0;
	displayList = glGenLists(1);
	glNewList(displayList, GL_COMPILE);
	glPushAttrib( GL_ENABLE_BIT | GL_LIGHTING_BIT );
	glDisable(GL_TEXTURE_2D);	
	GLfloat mat_spec[] = { 0.25, 0.25, 0.25, 1.0 };
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_spec);
	glColor3f(0.2,0.2,0.2);
	glutSolidSphere(radius,20,20);	
	glPopAttrib();
	glEndList();
}

CBadBall::~CBadBall()
{
	if (displayList != 0)
		glDeleteLists(displayList, 1);
}

void CBadBall::Draw()
{
	glPushMatrix();
	glTranslatef(position.x(),position.y(),position.z());
	glCallList(displayList);
	glPopMatrix();
}

void CBadBall::Update(float elapsed)
{
	CVector3d targetPos = target->GetPosition();

	CVector3d dist2ball = targetPos - position;
	// Only go after the target if its fairly close
	if ( dist2ball.Length() < leash*1.5f || (targetPos - anchor).Length() < leash*1.5f )
		velocity += dist2ball.UnitLength() * 0.008f;
	velocity *= 0.96f;

	velocity.SetY(0);
	CVector3d oldposition = position;
	position += velocity;
	CVector3d	dist2anchor = anchor - position;
	if (fabs(dist2anchor.x()) > leash)
		position.SetX(oldposition.x());
	if (fabs(dist2anchor.z()) > leash)
		position.SetZ(oldposition.z());

	bound->SetCenter(position);

	// Check for collision with player
	CVector3d intersection, normal;
	if (bound->CheckSphereCollision(targetPos, target->GetRadius(), intersection, normal))
	{
		target->ApplyForce(normal*20);
		velocity = -normal*0.1;
		position = intersection-normal*radius;
	}
}


