#ifndef H_BALL_H
#define H_BALL_H

#include <GL/glut.h>
#include "Vector3d.h"
#include "Texture.h"
#include "World.h"
#include "ParticleSystem.h"

class CWorld;

class CBall
{
public:
	CBall();
	~CBall();

	void Reset();
	void Init();
	void Draw();
	void Update(float elapsed);

	const CVector3d& GetPosition();
	const CVector3d& GetSafePosition();
	float GetRadius();
	float GetGroundHeight();
	void SetForce(const CVector3d& f);
	void ApplyForce(const CVector3d& f);
	void SetWorld(CWorld* world);
	void SetDone(bool d);
	void SetBall(const CTexture* tex, float, float, float);
	bool IsDone();

protected:
	float radius;
	CVector3d force;
	CVector3d externalForce;
	CVector3d velocity;
	CVector3d position;
	CVector3d safePosition;
	double rotation[16];
	float groundHeight;
	unsigned int displayList;

	GLUquadricObj *sphere;
	CWorld* world;

	CParticleSystem aura;

	bool done;

	const CTexture *baseTexture;
	CTexture reflectTexture;
	CTexture particleTexture;

	void CalculateRotation();
	void CheckCollisions();	
};

#endif
