#include "BallSelector.h"
#include <stdio.h>

float light[] = {40.0f, 40.0f, 0.0, 1.0};

extern int windowWidth;
extern int windowHeight;

void PrintText(char *string, int x, int y)
{
	glRasterPos2i(x, y);

	while(*string)
		glutBitmapCharacter( GLUT_BITMAP_TIMES_ROMAN_24, *string++);
}

CBallSelector::CBallSelector()
{
	sphere = 0;
	baseBallList = 0;
	envBallList = 0;
	ball = 0;
}

CBallSelector::~CBallSelector()
{
	if (sphere)
		gluDeleteQuadric(sphere);

	ballTexture[0].Delete();
	ballTexture[1].Delete();
	ballTexture[2].Delete();
	reflectTexture.Delete();
	floorTexture.Delete();
	shadowTexture.Delete();
	titleTexture.Delete();

	if (baseBallList != 0)
		glDeleteLists(baseBallList, 1);
	if (envBallList != 0)
		glDeleteLists(envBallList, 1);
}

void CBallSelector::Init()
{
	Reset();

	if (!sphere)
	{
		sphere = gluNewQuadric();
		gluQuadricTexture(sphere, GL_TRUE);
		gluQuadricNormals(sphere, GLU_SMOOTH);
	}

	ballTexture[0].LoadBitmapFile("blueplasma.bmp");
	ballTexture[1].LoadBitmapFile("ball.bmp");
	ballTexture[2].LoadBitmapFile("fire.bmp");
	reflectTexture.LoadBitmapFile("light.bmp");

	floorTexture.LoadBitmapFile("selection.bmp");
	shadowTexture.LoadBitmapFile("particle.bmp");
	titleTexture.LoadBitmapFile("title.bmp");

	// A ball with just the base texture, no sphere map
	baseBallList = glGenLists(1);
	glNewList(baseBallList, GL_COMPILE);
	//glColor4f( 0.7, 0.7, 0.7, 1.0f );	
	glDepthMask(GL_FALSE);
	gluSphere(sphere, 2, 32, 32 );
	glDepthMask(GL_TRUE);
	glEndList();

	// A ball with base texture and sphere map
	envBallList = glGenLists(1);
	glNewList(envBallList, GL_COMPILE);
	glCallList(baseBallList);	
	reflectTexture.Bind();
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	//glColor4f( 1.0, 1.0, 1.0, 0.55 );	
	glEnable(GL_TEXTURE_GEN_S); 
	glEnable(GL_TEXTURE_GEN_T);
	gluSphere(sphere, 2, 32, 32 );
	glDisable(GL_TEXTURE_GEN_S); 
	glDisable(GL_TEXTURE_GEN_T);
	glDisable(GL_BLEND);
	glEndList();

	CalculateShadowMatrix();		
}

void CBallSelector::SetBall(CBall* b)
{
	ball = b;
	ball->SetBall(&ballTexture[0], 0.0f, 0.0f, 1.0f);
}

void CBallSelector::Reset()
{
	done = false;

	selected = -1;
	alpha = 0.0f;
	
	balls[0].position = CVector3d(6,2,3.5);
	balls[1].position = CVector3d(-6,2,3.5);
	balls[2].position = CVector3d(0,2,-6.8);

	balls[0].velocity = balls[0].position.UnitLength();
	balls[1].velocity = balls[1].position.UnitLength();
	balls[2].velocity = balls[2].position.UnitLength();
	balls[0].velocity.SetY(0);
	balls[1].velocity.SetY(0);
	balls[2].velocity.SetY(0);
	
}

void CBallSelector::Draw()
{
	int i;
	float color[3];
	double angle[3];
	CVector3d arbitraryAxis[3];

	glPushAttrib(GL_ENABLE_BIT);

	glClear( GL_STENCIL_BUFFER_BIT );
	glColorMask(0,0,0,0);
	glDepthMask(0);
	glEnable(GL_STENCIL_TEST);
	glStencilFunc(GL_ALWAYS, 1, 0xFFFFFFFF );
	glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);

	glBegin(GL_QUADS);
		glNormal3f(0.0f, 1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-20.0,0,-20.0);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-20.0,0,20.0);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(20.0,0,20.0);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(20.0,0,-20.0);
	glEnd();

	glColorMask(1,1,1,1);
	glDepthMask(1);
	glStencilFunc(GL_EQUAL, 1, 0xFFFFFFFF );
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
  
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	// Draw reflected balls first
	glPushMatrix();
	glCullFace(GL_FRONT);
	glScalef(1.0,-1.0,1.0);
	glLightfv( GL_LIGHT0, GL_POSITION, light );

	for (i = 0; i < 3; i++)
	{
		glPushMatrix();
		glTranslatef(balls[i].position.x(), balls[i].position.y(), balls[i].position.z());

		arbitraryAxis[i].Set( balls[i].position.z(), 0, -balls[i].position.x() );
		float length = arbitraryAxis[i].Length();
		if (length != 0.0f)
		{
			angle[i] = 30.0f * length;
			arbitraryAxis[i] /= length;	
			glRotatef( angle[i], arbitraryAxis[i].x(), arbitraryAxis[i].y(), arbitraryAxis[i].z() );
		}
		color[i] = (1.0f - balls[i].position.Length()*0.05f);
		glColor4f( color[i], color[i], color[i], 0.55f );	
		ballTexture[i].Bind();
		glCallList(baseBallList);
		glPopMatrix();
	}

	glCullFace(GL_BACK);
	glPopMatrix();
	glLightfv( GL_LIGHT0, GL_POSITION, light );
	glDisable( GL_STENCIL_TEST );

	// Draw the blended floor
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_SRC_COLOR);
	glColor4f(1.0,1.0,1.0,0.75);
	glEnable(GL_POLYGON_OFFSET_FILL);
	glPolygonOffset(1.0, 1.0);
  floorTexture.Bind();	
	glBegin(GL_QUADS);
		glNormal3f(0.0f, 1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-20.0,0,-20.0);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-20.0,0,20.0);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(20.0,0,20.0);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(20.0,0,-20.0);
	glEnd();
	glDisable(GL_BLEND);
	glDisable(GL_POLYGON_OFFSET_FILL);
	
	// Draw the real balls
	for (i = 0; i < 3; i++)
	{
		glPushMatrix();
		glTranslatef(balls[i].position.x(), balls[i].position.y(), balls[i].position.z());
		glRotatef( angle[i], arbitraryAxis[i].x(), arbitraryAxis[i].y(), arbitraryAxis[i].z() );
		glColor4f( color[i], color[i], color[i], 0.55f );	
		ballTexture[i].Bind();
		glCallList(envBallList);
		glPopMatrix();
	}	

	glEnable(GL_TEXTURE_2D);
	shadowTexture.Bind();
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);

	glPushMatrix();
	float size = 3.5f;
	glColor3f(1.0,1.0,1.0);
	glMultMatrixf(shadow);

	for (int i = 0; i < 3; i++)
	{
		glPushMatrix();
		glTranslatef(balls[i].position.x(), balls[i].position.y(), balls[i].position.z());
		glBegin(GL_QUADS);
			glNormal3f(0.0f, 1.0f, 0.0f);
			glTexCoord2f(0.0f, 0.0f); glVertex3f(-size,0,-size);
			glTexCoord2f(0.0f, 1.0f); glVertex3f(-size,0,size);
			glTexCoord2f(1.0f, 1.0f); glVertex3f(size,0,size);
			glTexCoord2f(1.0f, 0.0f); glVertex3f(size,0,-size);
		glEnd();
		glPopMatrix();
	}
	glPopMatrix();

	glPushMatrix();
	glLoadIdentity();
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glColor4f(0.0,0.0,0.0,1.0);
	glBegin(GL_TRIANGLE_STRIP);
	glVertex3f(-2.5,1.6f,-5);
	glVertex3f(-2.5,1,-5);
	glVertex3f(2.5,1.6f,-5);
	glVertex3f(2.5,1,-5);
	glEnd();
	glColor4f(1.0,1.0,1.0,1.0);
	glEnable(GL_TEXTURE_2D);	
	titleTexture.Bind();
	glBegin(GL_TRIANGLE_STRIP);
	glTexCoord2f(0,1); glVertex3f(-1.0,1.55f,-5);
	glTexCoord2f(0,0.45f); glVertex3f(-1.0,1.0f,-5);
	glTexCoord2f(1,1); glVertex3f(1.0,1.55f,-5);
	glTexCoord2f(1,0.45f); glVertex3f(1.0,1.0f,-5);
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0, windowWidth, 0, windowHeight);
	glScalef(1, -1, 1);
	glTranslatef(0, -windowHeight, 0);

	if (selected == -1)
	{
		int x = (windowWidth>>1) - 90;
		glDepthMask(0);
		glColor4f(0.0f,0.0f,0.0f,alpha);
		PrintText( "Select your marble", x+2, windowHeight-30 );
		glColor4f(1.0f,1.0f,1.0f,alpha);
		PrintText( "Select your marble", x, windowHeight-32 );
		glDepthMask(1);
	}
		
	glColor4f(0.0f,0.0f,0.0f,1.0f-alpha);
	glBegin(GL_TRIANGLE_STRIP);
	glVertex3f(0,0,0);
	glVertex3f(0,windowHeight,0);
	glVertex3f(windowWidth,0,0);
	glVertex3f(windowWidth,windowHeight,0);
	glEnd();
	glDisable(GL_BLEND);
	glColor4f(0.0f,0.0f,0.0f,1.0f);
	glPopMatrix();

	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();


	glPopAttrib();

	Update();
}

void CBallSelector::Update()
{
	if (selected != -1)
	{
		for (int i = 0; i < 3; i++)
		{
			balls[i].position += balls[i].velocity * 0.12f;
			if (selected == i && balls[i].position.LengthSquared() <= 20)
			{
				balls[i].velocity *= 0.985f;
				if (balls[i].velocity.LengthSquared() <= 0.0004f)
				{
					done = true;
				}
			}
			else if (selected != i && balls[i].position.LengthSquared() >= 2000.0f)
				balls[i].velocity.Zero();
		}
	}
	if (!done && alpha < 1.0f)
	{
		alpha += 0.008f;
		if (alpha >= 1.0f)
			alpha = 1.0f;
	}
	else if (done && alpha > 0.0f)
	{
		alpha -= 0.008f;
		if (alpha <= 0.0f)
			alpha = 0.0f;
	}
}

void CBallSelector::CalculateShadowMatrix()
{
	float plane[] = {0.0f, 1.0f, 0.0, 0.0};
	float dot = plane[0] * light[0] + plane[1] * light[1] + plane[2] * light[2] + plane[3] * light[3];
	shadow[0] = dot - light[0]*plane[0];
	shadow[4] = - light[0]*plane[1];
	shadow[8] = - light[0]*plane[2];
	shadow[12] = - light[0]*plane[3];

	shadow[1] = - light[1]*plane[0];
	shadow[5] = dot - light[1]*plane[1];
	shadow[9] = - light[1]*plane[2];
	shadow[13] = - light[1]*plane[3];

	shadow[2] = - light[2]*plane[0];
	shadow[6] = - light[2]*plane[1];
	shadow[10] = dot - light[2]*plane[2];
	shadow[14] = - light[2]*plane[3];

	shadow[3] = - light[3]*plane[0];
	shadow[7] = - light[3]*plane[1];
	shadow[11] = - light[3]*plane[2];
	shadow[15] = dot - light[3]*plane[3];
}

void CBallSelector::ProcessClick(int x, int y)
{
	if (selected != -1)
		return;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_DITHER);
	glDisable(GL_LIGHTING);
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);

	for (int i = 0; i < 3; i++)
	{
		glPushMatrix();
		glTranslatef(balls[i].position.x(), balls[i].position.y(), balls[i].position.z());
		glColor3ub( (i == 0) ? 255 : 0, (i == 1) ? 255 : 0, (i == 2) ? 255 : 0);
		gluSphere(sphere, 2, 20, 20 );
		glPopMatrix();
	}

	GLubyte pixel[3];
	glReadPixels(x,y,1,1,GL_RGB,GL_UNSIGNED_BYTE,pixel);

	if (pixel[0] == 255)
	{
		selected = 0;
		ball->SetBall(&ballTexture[0], 0.0f, 0.0f, 1.0f);
		//printf("1st\n");
	}
	else if (pixel[1] == 255)
	{
		selected = 1;
		ball->SetBall(&ballTexture[1], 1.0f, 1.0f, 0.5f);
		//printf("2nd\n");
	}
	else if(pixel[2] == 255)
	{
		selected = 2;
		ball->SetBall(&ballTexture[2], 1.0f, 0.0f, 0.0f);
		//printf("3rd\n");
	}

	if (selected != -1)
		balls[selected].velocity = -balls[selected].velocity*0.5f;
	
	glPopAttrib();
}
