#include <GL/glut.h>
#include <math.h>
#include "Camera.h"

// Some math that we need
const double pi = 3.1415926538;
const double piOver180 = pi/180.0;

const float minDistance = 10.0f;
const float maxDistance = 45.0f;

float rads(float deg)
{
	return deg*piOver180;
}

CCamera::CCamera()
{
	target = NULL;
	Reset();
}

void CCamera::Reset()
{
	phi = 60.0;
	theta = 300.0;
	distance = 40.0;

	viewHeight = newHeight = 0.0f;
}

void CCamera::SetDistance(float dist)
{
	distance = dist;

	if (distance > maxDistance)
		distance = maxDistance;
	else if (distance < minDistance)
		distance = minDistance;
}

void CCamera::Zoom(float delta)
{
	distance += delta*0.01;

	if (distance > maxDistance)
		distance = maxDistance;
	else if (distance < minDistance)
		distance = minDistance;
}

void CCamera::Rotate(float delta)
{
	theta += delta*0.0075;
	
	if (theta > 360.0f)
		theta -= 360.0f;
	else if (theta < 0.0f)
		theta += 360.0f;
}

void CCamera::Tilt(float delta)
{
	phi += delta*0.005;
	
	if (phi > 62.5f)
		phi = 62.5f;
	else if (phi < 30.0f)
		phi = 30.0f;
}

void CCamera::SetTilt(float angle)
{
	phi = angle;
	if (phi > 62.5f)
		phi = 62.5f;
	else if (phi < 30.0f)
		phi = 30.0f;
}

float CCamera::GetRotationDegrees() const
{
	return rads(theta);
}

void CCamera::LookAt(bool center)
{
	CVector3d viewPoint, position;
	
	if (center)
	{
		viewPoint.Zero();
	}
	else
	{
		if (target != NULL)
			viewPoint = target->GetPosition();
		else
			viewPoint.Zero();

		newHeight = target->GetGroundHeight();
		
		if (viewHeight != newHeight)
		{
			float old = viewHeight;
			viewHeight += (newHeight-viewHeight)*0.005f;
			if ((newHeight > old && viewHeight > newHeight) || (newHeight < old && viewHeight < newHeight) )
				viewHeight = newHeight;
		}

		viewPoint.SetY(viewHeight);
	}

	position = viewPoint + 
		CVector3d( distance*sin(rads(theta))*sin(rads(phi)), distance*cos(rads(phi)), distance*cos(rads(theta))*sin(rads(phi)) );

	gluLookAt( position.x(), position.y(), position.z(),
		viewPoint.x(), viewPoint.y(), viewPoint.z(),
		0, 1, 0 );
}

void CCamera::SetTarget(CBall *ball)
{
	target = ball;
}
