#include <GL/glut.h>
#include <stdio.h>
#include "Clock.h"

CClock::CClock()
{
	Reset();
}

void CClock::Reset()
{
	time = 0.0f;
	started = false;
	startTime = -1;
	sprintf( str, "%.1f", time );
}

void CClock::PrintText(char *string, int x, int y)
{
	glRasterPos2i(x, y);

	while(*string)
		glutBitmapCharacter( GLUT_BITMAP_TIMES_ROMAN_24, *string++);
}

void CClock::Start()
{
	started = true;
}

void CClock::Stop()
{
	started = false;
}

bool CClock::IsRunning() const
{
	return started;
}

void CClock::Display(int windowWidth, int windowHeight)
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);

	if (started)
	{
		curTime = glutGet(GLUT_ELAPSED_TIME);
		if (startTime != -1)
		{
			long elapsedTime = curTime - startTime;
			// Only update once per tenth of a second
			if (elapsedTime >= 100)
			{
				time += elapsedTime * 0.001f;
				startTime = curTime;
				sprintf( str, "%.1f", time );
			}
		}
		else
			startTime = curTime;
	}

	// Need to set up an orthographic projection
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);	
	glPushMatrix();
	glLoadIdentity();

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0, windowWidth, 0, windowHeight);
	glScalef(1, -1, 1);
	glTranslatef(0, -windowHeight, 0);

	int x = (windowWidth>>1) - 40;

	glColor4f(0.4f,0.4f,0.4f,0.5f);
	glEnable(GL_BLEND);
	glDepthMask(0);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glBegin(GL_QUADS);
	glVertex3f(x,4,0);
	glVertex3f(x,45,0);
	glVertex3f(x+80,45,0);
	glVertex3f(x+80,4,0);
	glEnd();
	glColor4f(1.0f,0.4f,0.4f,0.5f);
	glBegin(GL_LINE_LOOP);
	glVertex3f(x,4,0);
	glVertex3f(x,45,0);
	glVertex3f(x+80,45,0);
	glVertex3f(x+80,4,0);
	glEnd();
	glDisable(GL_BLEND);

	int textX = x+25;
	if (time >= 100.0f)
		textX -= 10;
	else if (time >= 10.0f)
		textX -= 5;

	glColor4f(0.0, 0.0, 0.0, 1.0f);
	PrintText( str, textX+1, 33 );
	glColor4f(1.0, 1.0, 1.0, 1.0f);
	PrintText( str, textX, 32 );
	glDepthMask(1);
	glPopMatrix();

	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();

	glPopAttrib();
}
