#include "Cube.h"
#include <GL/glut.h>
#include "BoundingBox.h"

CCube::CCube(const CVector3d& pos, const CVector3d& size, const CTexture* tex, float xTex, float yTex)
: CObject(pos)
{
	float xSize = size.x();
	float ySize = size.y();
	float zSize = size.z();

	pts[0] = pos + CVector3d(-xSize, ySize, zSize);
	pts[1] = pos + CVector3d(-xSize, -ySize, zSize);
	pts[2] = pos + CVector3d(xSize, -ySize, zSize);
	pts[3] = pos + CVector3d(xSize, ySize, zSize);
	pts[4] = pos + CVector3d(xSize, ySize, -zSize);
	pts[5] = pos + CVector3d(xSize, -ySize, -zSize);
	pts[6] = pos + CVector3d(-xSize, -ySize, -zSize);
	pts[7] = pos + CVector3d(-xSize, ySize, -zSize);

	bound = new CBoundingBox(pos, size);

	rightAxis = CVector3d(1.0f,0.0f,0.0f);
	upAxis = CVector3d(0.0f,1.0f,0.0f);
	depthAxis = CVector3d(0.0f,0.0f,-1.0f);

	texture = tex;
	s = xTex;
	t = yTex;
}

CCube::CCube( const CVector3d& p1, const CVector3d& p2, const CVector3d& p3, float depth, const CTexture* tex , float xTex, float yTex)
{
	rightAxis = (p3-p2);
	upAxis = (p1-p2);

	float rightLength = rightAxis.Length();
	float upLength = upAxis.Length();
	
	// normalize the axes
	if (rightLength != 0.0f)
		rightAxis /= rightLength;
	if (upLength != 0.0f)
		upAxis /= upLength;

	depthAxis = CVector3d::CrossProduct(upAxis, rightAxis);
	position = ( p3+p1 + (depthAxis*depth) ) / 2.0f;
  
	CVector3d size(rightLength/2.0f, upLength/2.0f, depth/2.0f);
	
	pts[0] = p1;
	pts[1] = p2;
	pts[2] = p3;
	pts[3] = p1 + rightAxis*rightLength;
	pts[4] = pts[3] + depthAxis*depth;
	pts[5] = p3 + depthAxis*depth;
	pts[6] = p2 + depthAxis*depth;
	pts[7] = p1 + depthAxis*depth;

	bound = new CBoundingBox(position, size, rightAxis, upAxis);

	texture = tex;
	s = xTex;
	t = yTex;
}

void CCube::Draw()
{
	glPushAttrib(GL_ENABLE_BIT);
	
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glColor4f(1.0,1.0,1.0,1.0);
	texture->Bind();
	glBegin(GL_QUADS);

	glNormal3fv(-depthAxis);
	glTexCoord2f(0,t);  glVertex3fv( pts[0] );
	glTexCoord2f(0,0);  glVertex3fv( pts[1] );
	glTexCoord2f(s,0); glVertex3fv( pts[2] );
	glTexCoord2f(s,t);  glVertex3fv( pts[3] );

	glNormal3fv(rightAxis);
	glTexCoord2f(0,t);  glVertex3fv( pts[3] );
	glTexCoord2f(0,0);  glVertex3fv( pts[2] );
	glTexCoord2f(s,0); glVertex3fv( pts[5] );
	glTexCoord2f(s,t);  glVertex3fv( pts[4] );

	glNormal3fv(depthAxis);
	glTexCoord2f(0,t);   glVertex3fv( pts[4] );
	glTexCoord2f(0,0);  glVertex3fv( pts[5] );
	glTexCoord2f(s,0); glVertex3fv( pts[6] );
	glTexCoord2f(s,t);  glVertex3fv( pts[7] );

	glNormal3fv(-rightAxis);
	glTexCoord2f(s,t);   glVertex3fv( pts[7] );
	glTexCoord2f(s,0);  glVertex3fv( pts[6] );
	glTexCoord2f(0,0); glVertex3fv( pts[1] );
	glTexCoord2f(0,t);  glVertex3fv( pts[0] );

	glNormal3fv(upAxis);
	glTexCoord2f(s,t);   glVertex3fv( pts[7] );
	glTexCoord2f(0,t);  glVertex3fv( pts[0] );
	glTexCoord2f(0,0); glVertex3fv( pts[3] );
	glTexCoord2f(s,0);  glVertex3fv( pts[4] );

	glEnd();
	
	glPopAttrib();
}

