#include "Floor.h"
#include <GL/glut.h>
#include "BoundingBox.h"

extern const float worldHeight = -100.0f;

CFloor::CFloor(const CVector3d& pos, float x, float z, const CTexture* tex1, const CTexture* tex2 )
: CObject(pos)
{
	float ySize = -worldHeight/2.0f;
	xSize = x;
	zSize = z;

	bound = new CBoundingBox(pos - CVector3d(0,ySize,0), CVector3d(x,ySize,zSize));

	floorTexture = tex1;
	sidingTexture = tex2;
}

void CFloor::Draw()
{
	glPushAttrib(GL_ENABLE_BIT);
	
	glPushMatrix();
	glTranslatef(position.x(), position.y(), position.z());
	
	glColor4f( 1.0, 1.0, 1.0, 1.0 );
	
	glEnable(GL_LIGHTING);	
	glEnable(GL_TEXTURE_2D);
	floorTexture->Bind();
	
	float s = xSize*0.2f;
	float t = zSize*0.2f;
	
	glBegin(GL_QUADS);
	glNormal3f(0.0f,1.0f,0.0f);
	glTexCoord2f(0,t);  glVertex3f( -xSize, 0.0f, -zSize );
	glTexCoord2f(0,0);  glVertex3f( -xSize, 0.0f, zSize );
	glTexCoord2f(s,0); glVertex3f( xSize, 0.0f, zSize );
	glTexCoord2f(s,t);  glVertex3f( xSize, 0.0f, -zSize );
	glEnd();

	sidingTexture->Bind();

	float h = worldHeight - position.y();
	float r = -h*0.1f;

	glColor4f(1.0,1.0,1.0,1.0f);
	glBegin(GL_QUADS);
	glNormal3f(0.0f,0.0f,1.0f);
	glTexCoord2f(0,0);   glVertex3f( -xSize, 0, zSize );
	glTexCoord2f(0,r);  glVertex3f( -xSize, h, zSize );
	glTexCoord2f(s,r); glVertex3f( xSize, h, zSize );
	glTexCoord2f(s,0);  glVertex3f( xSize, 0, zSize );

	glNormal3f(1.0f,0.0f,0.0f);
	glTexCoord2f(0,0);   glVertex3f( xSize, 0, zSize );
	glTexCoord2f(0,r);  glVertex3f( xSize, h, zSize );
	glTexCoord2f(t,r); glVertex3f( xSize, h, -zSize );
	glTexCoord2f(t,0);  glVertex3f( xSize, 0, -zSize );

	glNormal3f(0.0f,0.0f,-1.0f);
	glTexCoord2f(0,0);   glVertex3f( xSize, 0, -zSize );
	glTexCoord2f(0,r);  glVertex3f( xSize, h, -zSize );
	glTexCoord2f(s,r); glVertex3f( -xSize, h, -zSize );
	glTexCoord2f(s,0);  glVertex3f( -xSize, 0, -zSize );

	glNormal3f(-1.0f,0.0f,0.0f);
	glTexCoord2f(0,0);   glVertex3f( -xSize, 0, -zSize );
	glTexCoord2f(0,r);  glVertex3f( -xSize, h, -zSize );
	glTexCoord2f(t,r); glVertex3f( -xSize, h, zSize );
	glTexCoord2f(t,0);  glVertex3f( -xSize, 0, zSize );
	glEnd();
	
	glPopMatrix();
	glPopAttrib();
}
