#include "Frustum.h"
#include <math.h>
#include <GL/glut.h>

void Side::Normalize()
{
	float invLength = 1.0f / sqrt(a*a + b*b + c*c);

	a *= invLength;
	b *= invLength;
	c *= invLength;
	d *= invLength;
}

void CFrustum::Calculate()
{ 
	float m[16];
	float p[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, m);
	glGetFloatv(GL_PROJECTION_MATRIX, p);

	float mult[16];

	mult[ 0] = m[ 0] * p[ 0] + m[ 1] * p[ 4] + m[ 2] * p[ 8] + m[ 3] * p[12];
	mult[ 1] = m[ 0] * p[ 1] + m[ 1] * p[ 5] + m[ 2] * p[ 9] + m[ 3] * p[13];
	mult[ 2] = m[ 0] * p[ 2] + m[ 1] * p[ 6] + m[ 2] * p[10] + m[ 3] * p[14];
	mult[ 3] = m[ 0] * p[ 3] + m[ 1] * p[ 7] + m[ 2] * p[11] + m[ 3] * p[15];

	mult[ 4] = m[ 4] * p[ 0] + m[ 5] * p[ 4] + m[ 6] * p[ 8] + m[ 7] * p[12];
	mult[ 5] = m[ 4] * p[ 1] + m[ 5] * p[ 5] + m[ 6] * p[ 9] + m[ 7] * p[13];
	mult[ 6] = m[ 4] * p[ 2] + m[ 5] * p[ 6] + m[ 6] * p[10] + m[ 7] * p[14];
	mult[ 7] = m[ 4] * p[ 3] + m[ 5] * p[ 7] + m[ 6] * p[11] + m[ 7] * p[15];

	mult[ 8] = m[ 8] * p[ 0] + m[ 9] * p[ 4] + m[10] * p[ 8] + m[11] * p[12];
	mult[ 9] = m[ 8] * p[ 1] + m[ 9] * p[ 5] + m[10] * p[ 9] + m[11] * p[13];
	mult[10] = m[ 8] * p[ 2] + m[ 9] * p[ 6] + m[10] * p[10] + m[11] * p[14];
	mult[11] = m[ 8] * p[ 3] + m[ 9] * p[ 7] + m[10] * p[11] + m[11] * p[15];

	mult[12] = m[12] * p[ 0] + m[13] * p[ 4] + m[14] * p[ 8] + m[15] * p[12];
	mult[13] = m[12] * p[ 1] + m[13] * p[ 5] + m[14] * p[ 9] + m[15] * p[13];
	mult[14] = m[12] * p[ 2] + m[13] * p[ 6] + m[14] * p[10] + m[15] * p[14];
	mult[15] = m[12] * p[ 3] + m[13] * p[ 7] + m[14] * p[11] + m[15] * p[15];

	plane[0].a = mult[ 3] - mult[ 0];
	plane[0].b = mult[ 7] - mult[ 4];
	plane[0].c = mult[11] - mult[ 8];
	plane[0].d = mult[15] - mult[12];
	plane[0].Normalize();

	plane[1].a = mult[ 3] + mult[ 0];
	plane[1].b = mult[ 7] + mult[ 4];
	plane[1].c = mult[11] + mult[ 8];
	plane[1].d = mult[15] + mult[12];
	plane[1].Normalize();

	plane[2].a = mult[ 3] + mult[ 1];
	plane[2].b = mult[ 7] + mult[ 5];
	plane[2].c = mult[11] + mult[ 9];
	plane[2].d = mult[15] + mult[13];
	plane[2].Normalize();

	plane[3].a = mult[ 3] - mult[ 1];
	plane[3].b = mult[ 7] - mult[ 5];
	plane[3].c = mult[11] - mult[ 9];
	plane[3].d = mult[15] - mult[13];
	plane[3].Normalize();

	plane[4].a = mult[ 3] - mult[ 2];
	plane[4].b = mult[ 7] - mult[ 6];
	plane[4].c = mult[11] - mult[10];
	plane[4].d = mult[15] - mult[14];
	plane[4].Normalize();

	plane[5].a = mult[ 3] + mult[ 2];
	plane[5].b = mult[ 7] + mult[ 6];
	plane[5].c = mult[11] + mult[10];
	plane[5].d = mult[15] + mult[14];
	plane[5].Normalize();

}

bool CFrustum::CheckSphere(const CVector3d& center, float radius) const
{
	for(int i = 0; i < 6; i++ )	
	{
		if( plane[i].a*center.x() + plane[i].b*center.y() + plane[i].c*center.z() + plane[i].d <= -radius )
			return false;
	}

	return true;
}

bool CFrustum::CheckBox( const CVector3d* pts ) const
{
	for(int i = 0; i < 6; i++ )
	{
		if (plane[i].a*pts[0].x() + plane[i].b*pts[0].y() + plane[i].c*pts[0].z() + plane[i].d < 0 && 
			plane[i].a*pts[1].x() + plane[i].b*pts[1].y() + plane[i].c*pts[1].z() + plane[i].d < 0 && 
			plane[i].a*pts[2].x() + plane[i].b*pts[2].y() + plane[i].c*pts[2].z() + plane[i].d < 0 && 
			plane[i].a*pts[3].x() + plane[i].b*pts[3].y() + plane[i].c*pts[3].z() + plane[i].d < 0 && 
			plane[i].a*pts[4].x() + plane[i].b*pts[4].y() + plane[i].c*pts[4].z() + plane[i].d < 0 && 
			plane[i].a*pts[5].x() + plane[i].b*pts[5].y() + plane[i].c*pts[5].z() + plane[i].d < 0 && 
			plane[i].a*pts[6].x() + plane[i].b*pts[6].y() + plane[i].c*pts[6].z() + plane[i].d < 0 && 
			plane[i].a*pts[7].x() + plane[i].b*pts[7].y() + plane[i].c*pts[7].z() + plane[i].d < 0)
		{
			return false;
		}
	}

	return true;
}

bool CFrustum::CheckPlane(const CVector3d* pts) const
{
	for(int i = 0; i < 6; i++ )
	{
		if (plane[i].a*pts[0].x() + plane[i].b*pts[0].y() + plane[i].c*pts[0].z() + plane[i].d < 0 && 
			plane[i].a*pts[1].x() + plane[i].b*pts[1].y() + plane[i].c*pts[1].z() + plane[i].d < 0 && 
			plane[i].a*pts[2].x() + plane[i].b*pts[2].y() + plane[i].c*pts[2].z() + plane[i].d < 0 && 
			plane[i].a*pts[3].x() + plane[i].b*pts[3].y() + plane[i].c*pts[3].z() + plane[i].d < 0)
		{
			return false;
		}
	}

	return true;
}


