#ifndef H_PARTICLESYSTEM_H
#define H_PARTICLESYSTEM_H

#include <GL/glut.h>
#include "Vector3d.h"
#include "Texture.h"
#include <stdlib.h>

#define HALF_RAND (RAND_MAX/2.0f)

class CParticle
{
 public:
	float life,fade;
	CVector3d position;
	CVector3d velocity;
	float r,g,b;
	float dr,dg,db;
	float size, dsize;
	bool exploding;

	void Draw();
};


class CParticleSystem
{	
 public:
	CParticleSystem();
	virtual ~CParticleSystem();

	void SetNumberOfParticles(int);

	void SetOriginRange(const CVector3d& r) { originRange = r; }
	void SetOrigin(const CVector3d& o) { origin = o; }
		
	void SetVelocity(const CVector3d& v) { velocity = v; }
	void SetVelocityRange(const CVector3d& r) { velocityRange = r; }
	void SetForce(const CVector3d& f) { force = force; }
	
	void SetLife(float l) { life = l; }
	void SetFade(float f) { fade = f; } 
	void SetFadeRange(float r) { fadeRange = r; }

	void SetColor(float start[], float end[]);
	void SetColor(float r1,float g1, float b1, float r2, float g2, float b2);

	void SetSize(float s1, float s2) { startSize = s1; endSize = s2; }
	void SetSize(float s) { startSize = endSize = s; }

	void SetEmissionRate(float r) { rate = 1.0f/r; }
	void SetTexture(const CTexture* t) { texture = t; }

	void Init(bool update = true);
	void Draw(float, bool update = true);
	void Flush();
	
 protected:
	virtual void UpdateParticle(int,float);
	void DrawParticle(int);
	virtual void UpdateAttributes(int, float);
	virtual void AdvanceParticleInList(int);
	virtual void DecrementLife(int,float);
	virtual void InitParticle(int);
	virtual void CleanUp();

	CParticle *particles;
	const CTexture* texture;

	int numberOfParticles, maxNumberParticles;
	float life, fade;	
	float fadeRange;
	float rate;

	CVector3d origin;
	CVector3d originRange;
	CVector3d velocity;
	CVector3d velocityRange;
	CVector3d force;

	float startColor[3];
	float endColor[3];
	float startSize, endSize;

	float elapsedTime;	
	CVector3d right, up;  // vectors used to billboard the particles
};

//**********************************************************************************
// CParticle
//**********************************************************************************

inline void CParticle::Draw() 
{
	glColor4f(r,g,b,life); 
	const float* p = position;
	glBegin(GL_TRIANGLE_STRIP);
	 glTexCoord2d(1, 1); glVertex3f(p[0]+size,p[1]+size,p[2]);		 
	 glTexCoord2f(0, 1); glVertex3f(p[0]-size,p[1]+size,p[2]);
	 glTexCoord2f(1, 0); glVertex3f(p[0]+size,p[1]-size,p[2]);		
	 glTexCoord2f(0, 0); glVertex3f(p[0]-size,p[1]-size,p[2]);
	glEnd();
	 
}

inline float frand()
{
	return ((float)rand() / (float)HALF_RAND - 1.0f);
}

#endif 
