#include "Torch.h"
#include "BoundingBox.h"

CTorch::CTorch(const CVector3d& pos, const CTexture* base, const CTexture* particle)
: CObject(pos)
{
	baseTexture = base;

	bound = new CBoundingBox(pos + CVector3d(0,2,0), CVector3d(0.25f,4.0f,0.25f));
	
	flame.SetNumberOfParticles(22);
	flame.SetEmissionRate(15);
	flame.SetColor(1.0,1.0,0.5,1.0,0.0,0.0);
	flame.SetLife(3.25f);
	flame.SetSize(1.00f,0.50f);
	flame.SetFade(2.5f);
	flame.SetFadeRange(0.25f);
	flame.SetOriginRange(CVector3d(0.05,0.05,0.05));
	flame.SetVelocity(CVector3d(0,2.5,0));
	flame.SetVelocityRange(CVector3d(0.5,0.5,0.5));
	flame.SetTexture(particle);
}

void CTorch::Draw()
{
	glPushAttrib(GL_ENABLE_BIT);
	glPushMatrix();

	glTranslatef(position.x(),position.y(),position.z());

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glColor4f(1.0,1.0,1.0,1.0);
	baseTexture->Bind();

	glBegin(GL_QUADS);
	glNormal3f(0.0,0.0,1.0);
	glTexCoord2f(0,1);  glVertex3f( -0.25f, 2, 0.25f );
	glTexCoord2f(0,0);  glVertex3f( -0.25f, -2, 0.25f );
	glTexCoord2f(1,0); glVertex3f( 0.25f, -2, 0.25f );
	glTexCoord2f(1,1);  glVertex3f( 0.25f, 2, 0.25f );

	glNormal3f(1.0,0.0,0.0);
	glTexCoord2f(0,1);  glVertex3f( 0.25f, 2, 0.25f );
	glTexCoord2f(0,0);  glVertex3f( 0.25f, -2, 0.25f );
	glTexCoord2f(1,0); glVertex3f( 0.25f, -2, -0.25f );
	glTexCoord2f(1,1);  glVertex3f( 0.25f, 2, -0.25f );

	glNormal3f(0.0,0.0,-1.0);
	glTexCoord2f(0,1);  glVertex3f( 0.25f, 2, -0.25f );
	glTexCoord2f(0,0);  glVertex3f( 0.25f, -2, -0.25f );
	glTexCoord2f(1,0); glVertex3f( -0.25f, -2, -0.25f );
	glTexCoord2f(1,1);  glVertex3f( -0.25f, 2, -0.25f );

	glNormal3f(-1.0,0.0,0.0);
	glTexCoord2f(0,1);  glVertex3f( -0.25f, 2, -0.25f );
	glTexCoord2f(0,0);  glVertex3f( -0.25f, -2, -0.25f );
	glTexCoord2f(1,0); glVertex3f( -0.25f, -2, 0.25f );
	glTexCoord2f(1,1);  glVertex3f( -0.25f, 2, 0.25f );

	glNormal3f(0.0,1.0,0.0);
	glTexCoord2f(0,1);  glVertex3f( -0.25f, 2, -0.25f );
	glTexCoord2f(0,0);  glVertex3f( -0.25f, 2, 0.25f );
	glTexCoord2f(1,0); glVertex3f( 0.25f, 2, 0.25f );
	glTexCoord2f(1,1);  glVertex3f( 0.25f, 2, -0.25f );
	glEnd();

	glTranslatef(0.0f,2.5f,0.0f);
	
	glDepthMask(0);
	glEnable(GL_BLEND);
	glDisable(GL_LIGHTING);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	flame.Draw(0.02f,true);
	glDepthMask(1);
	
	glPopMatrix();
	glPopAttrib();
}
