#include <GL/glut.h>
#include "World.h"
#include "Cube.h"
#include "BadBall.h"
#include "Plane.h"
#include "MovingFloor.h"
#include "Torch.h"

CWorld::CWorld()
{
	ball = 0;
	floorDisplayList = 0;
	endPosition = 0;
}

CWorld::~CWorld()
{
	tileTexture[0].Delete();
	tileTexture[1].Delete();
	floorTexture[0].Delete();
	floorTexture[1].Delete();
	woodTexture.Delete();
	barrierTexture.Delete();
	particleTexture.Delete();
	
	std::list<CBoundingVolume*>::iterator bound;
	for (bound = bounds.begin(); bound != bounds.end(); bound++)
		delete (*bound);

	std::list<CObject*>::iterator object;
	for (object = objects.begin(); object != objects.end(); object++)
		delete (*object);

	std::list<CFloor*>::iterator floor;
	for (floor = floors.begin(); floor != floors.end(); floor++)
		delete (*floor);

	if (endPosition)
		delete endPosition;

	if (floorDisplayList != 0)
		glDeleteLists(floorDisplayList, 1);
}

void CWorld::Init()
{
	tileTexture[0].LoadBitmapFile("blacktile.bmp");
	tileTexture[1].LoadBitmapFile("whitetile.bmp");
	floorTexture[0].LoadBitmapFile("marble.bmp");
	floorTexture[1].LoadBitmapFile("whitemarble.bmp");
	woodTexture.LoadBitmapFile("boxwood.bmp");
	barrierTexture.LoadBitmapFile("barrier.bmp");
	particleTexture.LoadBitmapFile("particle.bmp");

	// Set up the world
	//startPosition.Set(-42.5,0,-30);
	startPosition.Set(-140,12.5f,-100);
	//startPosition.Set(55, 0, -142.5);
	//startPosition.Set(65, 0, 45);
	
	AddFloor( new CFloor( CVector3d(-95,10,-112.5), 20, 2.5, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-95,10,-87.5), 20, 2.5, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-115,10,-100), 5, 10, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CPlane( CVector3d(-130.0f, 12.5, -105), CVector3d(-130.0f, 12.5, -95), CVector3d(-120.0f, 10.0, -95.0f), &floorTexture[0] ) );
	AddObject( new CCube( CVector3d(-110.25f,11.75f,-100), CVector3d(0.25f,1.75f,10.0f), &barrierTexture, 4 ));	
	AddFloor( new CFloor( CVector3d(-140,12.5f,-100), 10, 10, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-65,10,-100), 15, 10, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CPlane( CVector3d(-60, 10, -90), CVector3d(-60, 5, -70), CVector3d(-50, 5, -70), &floorTexture[0], 1.0f, 2.0f ) );
	AddFloor( new CFloor( CVector3d(-50,5,-65), 10, 5, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CPlane( CVector3d(-45, 5, -60), CVector3d(-45, 0, -50), CVector3d(-40, 0, -50), &floorTexture[0], 0.5f, 1.5f ) );
	
	AddFloor( new CFloor( CVector3d(-42.5,0,-30), 7.5, 20, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-45,0,-7.5), 10, 2.5f, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-50,0,-2.5), 10, 2.5f, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-55,0,2.5), 10, 2.5f, &floorTexture[0], &tileTexture[0] ) );

	AddFloor( new CFloor( CVector3d(-60,0,8.75f), 5, 3.75f, &woodTexture, &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-60,0,25), 12.5, 12.5, &floorTexture[1], &tileTexture[0] ) );
	AddObject( new CBadBall( CVector3d(-60,1.2f,25), 1.3f, 8.5, ball) );
	AddFloor( new CFloor( CVector3d(-60,0,41.25f), 5, 3.75f, &woodTexture, &tileTexture[0] ) );

	AddFloor( new CFloor( CVector3d(-55,0,47.5), 10, 2.5f, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-50,0,52.5), 10, 2.5f, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(-47.5,0,57.5), 7.5f, 2.5f, &floorTexture[0], &tileTexture[0] ) );

	AddObject( new CCube( CVector3d(-49.75f, 1.75f, -32.5f), CVector3d(0.25f,1.75f,17.5f), &barrierTexture, 5 ));	
	AddObject( new CCube( CVector3d(-47.25, 1.75f, -49.75f), CVector3d(2.25f,1.75f,0.25f), &barrierTexture, 1 ));	
	AddObject( new CCube( CVector3d(-35.25f, 1.75f, -32.5f), CVector3d(0.25f,1.75f,17.5f), &barrierTexture, 5 ));	
	AddObject( new CCube( CVector3d(-37.75, 1.75f, -49.75f), CVector3d(2.25f,1.75f,0.25f), &barrierTexture, 1 ));	


	AddFloor( new CFloor( CVector3d(5,-10,55), 5, 5, &floorTexture[0], &tileTexture[0]) );
	AddObject( new CPlane( CVector3d(-40, 0, 50), CVector3d(-40, 0, 60), CVector3d(0, -10, 60), &floorTexture[0], 4.0f, 1.0f ) );
	AddObject( new CPlane( CVector3d(6.5, 0.5f, 9), CVector3d(3.5, 0.5f, 9), CVector3d(3.5, -10, 51), &woodTexture, 4.0f, 1.0f ) );
	AddFloor( new CFloor( CVector3d(5,0,-20), 20, 30, &floorTexture[0], &tileTexture[0]) );
	AddObject( new CCube( CVector3d(-14.75f, 1.75f, -20), CVector3d(0.25f,1.75f,30.0f), &barrierTexture, 5 ));	
	AddObject( new CCube( CVector3d(24.75f, 1.75f, -20), CVector3d(0.25f,1.75f,30.0f), &barrierTexture, 5 ));	

	AddObject( new CCube( CVector3d(-9,1.5,0), CVector3d(5.0f,1.5f,2.0f), &woodTexture ));	
	AddObject( new CCube( CVector3d(5,1.5,0), CVector3d(5.0f,1.5f,2.0f), &woodTexture ));	
	AddObject( new CCube( CVector3d(19,1.5,0), CVector3d(5.0f,1.5f,2.0f), &woodTexture ));	

	AddObject( new CCube( CVector3d(-2.5f,1.5,-10), CVector3d(5.0f,1.5f,2.0f), &woodTexture ));	
	AddObject( new CCube( CVector3d(12.5f,1.5,-10), CVector3d(5.0f,1.5f,2.0f), &woodTexture ));	

	AddObject( new CCube( CVector3d(-9,1.5,-20), CVector3d(5.0f,1.5f,2.0f), &woodTexture ));	
	AddObject( new CCube( CVector3d(5,1.5,-20), CVector3d(5.0f,1.5f,2.0f), &woodTexture ));	
	AddObject( new CCube( CVector3d(19,1.5,-20), CVector3d(5.0f,1.5f,2.0f), &woodTexture ));	

	AddObject( new CBadBall( CVector3d(5,1.2f,-75), 1.3f, 10.75f, ball) );
	AddFloor( new CFloor( CVector3d(5,0,-75), 15, 15, &floorTexture[1], &tileTexture[1]) );
	AddObject( new CPlane( CVector3d(0,0,-45), CVector3d(10,0,-45), CVector3d(10,1.5,-50), &barrierTexture ) );
	AddObject( new CMovingFloor( CVector3d(5,0.0f,-95), CVector3d(5,8,-95), 5, 5, &floorTexture[1], &tileTexture[1]) );
	AddFloor( new CFloor( CVector3d(5,16,-105), 5, 5, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CPlane( CVector3d(5, 16, -110), CVector3d(10, 16, -110), CVector3d(10, 8, -140), &floorTexture[0], 6.0f, 1.0f ) );
	AddFloor( new CFloor( CVector3d(7.5f, 8, -142.5f), 2.5f, 2.5f, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CPlane( CVector3d(10, 8, -145), CVector3d(10, 8, -140), CVector3d(40, 0, -140), &floorTexture[0], 6.0f, 1.0f ) );
	AddFloor( new CFloor( CVector3d(57.5, 0, -142.5), 17.5, 5, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CPlane( CVector3d(75,0,-147.5), CVector3d(75,0,-137.5), CVector3d(79,2,-137.5), &barrierTexture ) );
	
	AddObject( new CCube( CVector3d(90, 6.0f, -110), CVector3d(90, 0.0f, -110), CVector3d(100, 0.0f, -100), 14.0f, &tileTexture[1], 1.5f, 1.5f ));	
	AddObject( new CCube( CVector3d(110, 4.0f, -120), CVector3d(110, 0.0f, -120), CVector3d(115, 0.0f, -115), 7.0f, &tileTexture[1] ));	
	AddObject( new CCube( CVector3d(110, 4.0f, -100), CVector3d(110, 0.0f, -100), CVector3d(115, 0.0f, -95), 7.0f, &tileTexture[1] ));	
	AddFloor( new CFloor( CVector3d(105, 0, -115), 15, 35, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CCube( CVector3d(105, 1.75f, -149.75f), CVector3d(15,1.75f,0.25f), &barrierTexture, 5 ));	
	AddObject( new CCube( CVector3d(105, 1.75f, -80.25f), CVector3d(15,1.75f,0.25f), &barrierTexture, 5 ));	
	AddObject( new CCube( CVector3d(119.75f, 1.75f, -135), CVector3d(0.25f,1.75f,14.5), &barrierTexture, 5 ));	
	AddObject( new CCube( CVector3d(119.75f, 1.75f, -90), CVector3d(0.25f,1.75f,9.5), &barrierTexture, 5 ));	
	AddFloor( new CFloor( CVector3d(70, 0, -82.5f), 20, 2.5f, &floorTexture[1], &tileTexture[1] ) );
	AddFloor( new CFloor( CVector3d(52.5f, 0, -75), 2.5f, 5, &floorTexture[1], &tileTexture[1] ) );

	AddFloor( new CFloor( CVector3d(65, 0, -67.5), 15, 2.5f, &floorTexture[1], &tileTexture[1] ) );
	AddFloor( new CFloor( CVector3d(77.5, 0, -55), 2.5, 10, &floorTexture[1], &tileTexture[1] ) );
	AddFloor( new CFloor( CVector3d(65, 0, -42.5), 15, 2.5f, &floorTexture[1], &tileTexture[1] ) );
	AddObject( new CPlane( CVector3d(50, 0, -40), CVector3d(50, 1.5f, -30), CVector3d(55, 1.5f, -30), &tileTexture[1], 0.5f, 1.0f ) );
	AddObject( new CPlane( CVector3d(50, 1.5f, -30), CVector3d(50, 0, -20), CVector3d(55, 0, -20), &tileTexture[1], 0.5f, 1.0f ) );
	AddObject( new CPlane( CVector3d(50, 0, -20), CVector3d(50, 1.5f, -10), CVector3d(55, 1.5f, -10), &tileTexture[1], 0.5f, 1.0f ) );
	AddObject( new CPlane( CVector3d(50, 1.5f, -10), CVector3d(50, 0, 0), CVector3d(55, 0, 0), &tileTexture[1], 0.5f, 1.0f ) );
	AddFloor( new CFloor( CVector3d(65, 0, 20), 15, 20, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CCube( CVector3d(62.5, 1.5f, 30), CVector3d(12.5f,1.5f,2.0f), &floorTexture[1]));	
	AddObject( new CCube( CVector3d(67.5, 1.5f, 20), CVector3d(12.5f,1.5f,2.0f), &floorTexture[1]));	
	AddObject( new CCube( CVector3d(62.5, 1.5f, 10), CVector3d(12.5f,1.5f,2.0f), &floorTexture[1]));	
	AddFloor( new CFloor( CVector3d(65, 0, 45), 10, 5, &floorTexture[0], &tileTexture[0] ) );
	AddFloor( new CFloor( CVector3d(65, 0, 55), 5, 5, &floorTexture[0], &tileTexture[0] ) );
	AddObject( new CPlane( CVector3d(60, 0, 80), CVector3d(65, -1, 80), CVector3d(65, -1, 60), &floorTexture[0], 4.0f, 1.0f ) );
	AddObject( new CPlane( CVector3d(70, 0, 60), CVector3d(65, -1, 60), CVector3d(65, -1, 80), &floorTexture[0], 4.0f, 1.0f ) );
	AddObject( new CTorch( CVector3d(-131,14.5f,-109), &woodTexture, &particleTexture ) );
	AddObject( new CTorch( CVector3d(-131,14.5f,-91), &woodTexture, &particleTexture ) );
	AddObject( new CTorch( CVector3d(56, 2, 45), &woodTexture, &particleTexture ) );
	AddObject( new CTorch( CVector3d(74, 2, 45), &woodTexture, &particleTexture ) );

	endPosition = new CBoundingBox( CVector3d(65, -1, 70), CVector3d(5-ball->GetRadius(), 5, 10-ball->GetRadius()) );	

	// Put static geometry in a display list
	floorDisplayList = glGenLists(1);
	glNewList(floorDisplayList, GL_COMPILE);
	std::list< CFloor* >::iterator floor;
	for (floor = floors.begin(); floor != floors.end(); floor++)
		(*floor)->Draw();
	/*std::list< CObject* >::iterator obj;
	for (obj = objects.begin(); obj != objects.end(); obj++)
	{
		if ((*obj)->IsStatic())
			(*obj)->Draw();
	}*/
	glEndList();
}

bool CWorld::IsBallDone()
{
	CVector3d intersection, normal;
	if (endPosition->CheckSphereCollision(ball->GetPosition(), ball->GetRadius(), intersection, normal))
	{
		// the Collision test returns the upAxis as the normal if we are inside
		// This is a bit of a hack to check whether we are completely inside the bounding box
		// since we know we can never hit the top of this box, 
		if (normal.y() == 1.0f)
			return true;
	}
	
	return false;
}

void CWorld::SetBall(CBall *b)
{
	ball = b;
}

void CWorld::AddObject(CObject *obj)
{
	objects.push_back( obj );

	if (obj->HasBoundingVolume())
		bounds.push_back( obj->GetBoundingVolume() );
}

void CWorld::AddFloor(CFloor *floor)
{
	floors.push_back( floor );

	if (floor->HasBoundingVolume())
		bounds.push_back( floor->GetBoundingVolume() );
}

void CWorld::DrawFloor()
{
	//glCallList(floorDisplayList);
	std::list< CFloor* >::iterator floor;
	for (floor = floors.begin(); floor != floors.end(); floor++)
	{
		if ((*floor)->GetBoundingVolume()->InFrustum(frustum))
			(*floor)->Draw();
	}
}

void CWorld::DrawObjects()
{
	frustum.Calculate();
	int i = 0;
	std::list< CObject* >::iterator obj;
	for (obj = objects.begin(); obj != objects.end(); obj++)
	{
		//if (!(*obj)->IsStatic())
		{
			if ((*obj)->GetBoundingVolume()->InFrustum(frustum))
			{
				(*obj)->Update(1.0f);
				(*obj)->Draw();
				i++;
			}
		}
	}

	//printf("%d\n", i);
}
